@@ -6,7 +6,7 @@ Generic Python packager for Android / iOS. Desktop later.
 
 '''
 
-__version__ = '1.2.0'
+__version__ = '1.3.0'
 
 import os
 import re
@@ -14,11 +14,12 @@ import sys
 import select
 import codecs
 import textwrap
+import warnings
 from buildozer.jsonstore import JsonStore
 from sys import stdout, stderr, exit
 from re import search
 from os.path import join, exists, dirname, realpath, splitext, expanduser
-from subprocess import Popen, PIPE
+from subprocess import Popen, PIPE, TimeoutExpired
 from os import environ, unlink, walk, sep, listdir, makedirs
 from copy import copy
 from shutil import copyfile, rmtree, copytree, move
@@ -27,7 +28,7 @@ from fnmatch import fnmatch
 from pprint import pformat
 
 from urllib.request import FancyURLopener
-from configparser import SafeConfigParser
+from configparser import ConfigParser
 try:
     import fcntl
 except ImportError:
@@ -112,7 +113,7 @@ class Buildozer:
         self.state = None
         self.build_id = None
         self.config_profile = ''
-        self.config = SafeConfigParser(allow_no_value=True)
+        self.config = ConfigParser(allow_no_value=True)
         self.config.optionxform = lambda value: value
         self.config.getlist = self._get_config_list
         self.config.getlistvalues = self._get_config_list_values
@@ -171,7 +172,6 @@ class Buildozer:
         self.info('Check application requirements')
         self.check_application_requirements()
 
-        self.info('Check garden requirements')
         self.check_garden_requirements()
 
         self.info('Compile platform')
@@ -253,7 +253,7 @@ class Buildozer:
 
     def cmd(self, command, **kwargs):
         # prepare the environ, based on the system + our own env
-        env = copy(environ)
+        env = environ.copy()
         env.update(self.environ)
 
         # prepare the process
@@ -269,15 +269,18 @@ class Buildozer:
         get_stderr = kwargs.pop('get_stderr', False)
         break_on_error = kwargs.pop('break_on_error', True)
         sensible = kwargs.pop('sensible', False)
+        run_condition = kwargs.pop('run_condition', None)
+        quiet = kwargs.pop('quiet', False)
 
-        if not sensible:
-            self.debug('Run {0!r}'.format(command))
-        else:
-            if type(command) in (list, tuple):
-                self.debug('Run {0!r} ...'.format(command[0]))
+        if not quiet:
+            if not sensible:
+                self.debug('Run {0!r}'.format(command))
             else:
-                self.debug('Run {0!r} ...'.format(command.split()[0]))
-        self.debug('Cwd {}'.format(kwargs.get('cwd')))
+                if isinstance(command, (list, tuple)):
+                    self.debug('Run {0!r} ...'.format(command[0]))
+                else:
+                    self.debug('Run {0!r} ...'.format(command.split()[0]))
+            self.debug('Cwd {}'.format(kwargs.get('cwd')))
 
         # open the process
         if sys.platform == 'win32':
@@ -297,9 +300,9 @@ class Buildozer:
 
         ret_stdout = [] if get_stdout else None
         ret_stderr = [] if get_stderr else None
-        while True:
+        while not run_condition or run_condition():
             try:
-                readx = select.select([fd_stdout, fd_stderr], [], [])[0]
+                readx = select.select([fd_stdout, fd_stderr], [], [], 1)[0]
             except select.error:
                 break
             if fd_stdout in readx:
@@ -322,7 +325,13 @@ class Buildozer:
             stdout.flush()
             stderr.flush()
 
-        process.communicate()
+        try:
+            process.communicate(
+                timeout=(1 if run_condition and not run_condition() else None)
+            )
+        except TimeoutExpired:
+            pass
+
         if process.returncode != 0 and break_on_error:
             self.error('Command failed: {0}'.format(command))
             self.log_env(self.ERROR, kwargs['env'])
@@ -337,10 +346,12 @@ class Buildozer:
                 self.error('raising an issue with buildozer itself.')
             self.error('In case of a bug report, please add a full log with log_level = 2')
             raise BuildozerCommandException()
+
         if ret_stdout:
             ret_stdout = b''.join(ret_stdout)
         if ret_stderr:
             ret_stderr = b''.join(ret_stderr)
+
         return (ret_stdout.decode('utf-8', 'ignore') if ret_stdout else None,
                 ret_stderr.decode('utf-8') if ret_stderr else None,
                 process.returncode)
@@ -349,7 +360,7 @@ class Buildozer:
         from pexpect import spawnu
 
         # prepare the environ, based on the system + our own env
-        env = copy(environ)
+        env = environ.copy()
         env.update(self.environ)
 
         # prepare the process
@@ -509,44 +520,10 @@ class Buildozer:
                  cwd=self.buildozer_dir)
 
     def check_garden_requirements(self):
-        '''Ensure required garden packages are available to be included.
-        '''
         garden_requirements = self.config.getlist('app',
-                'garden_requirements', '')
-
-        # have we installed the garden packages?
-        if exists(self.gardenlibs_dir) and \
-                self.state.get('cache.gardenlibs', '') == garden_requirements:
-            self.debug('Garden requirements already installed, pass')
-            return
-
-        # we're going to reinstall all the garden libs.
-        self.rmdir(self.gardenlibs_dir)
-
-        # but if we don't have requirements, or if the user removed everything,
-        # don't do anything.
-        if not garden_requirements:
-            self.state['cache.gardenlibs'] = garden_requirements
-            return
-
-        self._ensure_virtualenv()
-        self.cmd('pip install Kivy-Garden==0.1.1', env=self.env_venv)
-
-        # recreate gardenlibs
-        self.mkdir(self.gardenlibs_dir)
-
-        for requirement in garden_requirements:
-            self._install_garden_package(requirement)
-
-        # save gardenlibs state
-        self.state['cache.gardenlibs'] = garden_requirements
-
-    def _install_garden_package(self, package):
-        self._ensure_virtualenv()
-        self.debug('Install garden package {} in buildozer_dir'.format(package))
-        self.cmd('garden install --app {}'.format(package),
-                env=self.env_venv,
-                cwd=self.buildozer_dir)
+            'garden_requirements', '')
+        if garden_requirements:
+            warnings.warn("`garden_requirements` settings is deprecated, use `requirements` instead", DeprecationWarning)
 
     def _ensure_virtualenv(self):
         if hasattr(self, 'venv'):
@@ -715,12 +692,11 @@ class Buildozer:
     def build_application(self):
         self._copy_application_sources()
         self._copy_application_libs()
-        self._copy_garden_libs()
         self._add_sitecustomize()
 
     def _copy_application_sources(self):
         # XXX clean the inclusion/exclusion algo.
-        source_dir = realpath(self.config.getdefault('app', 'source.dir', '.'))
+        source_dir = realpath(expanduser(self.config.getdefault('app', 'source.dir', '.')))
         include_exts = self.config.getlist('app', 'source.include_exts', '')
         exclude_exts = self.config.getlist('app', 'source.exclude_exts', '')
         exclude_dirs = self.config.getlist('app', 'source.exclude_dirs', '')
@@ -730,6 +706,12 @@ class Buildozer:
                                                '')
         app_dir = self.app_dir
 
+        include_exts = [ext.lower() for ext in include_exts]
+        exclude_exts = [ext.lower() for ext in exclude_exts]
+        exclude_dirs = [dir.lower() for dir in exclude_dirs]
+        exclude_patterns = [pat.lower() for pat in exclude_patterns]
+        include_patterns = [pat.lower() for pat in include_patterns]
+
         self.debug('Copy application source from {}'.format(source_dir))
 
         rmtree(self.app_dir)
@@ -751,7 +733,7 @@ class Buildozer:
                 for exclude_dir in exclude_dirs:
                     if exclude_dir[-1] != '/':
                         exclude_dir += '/'
-                    if filtered_root.startswith(exclude_dir.lower()):
+                    if filtered_root.startswith(exclude_dir):
                         is_excluded = True
                         break
 
@@ -795,7 +777,7 @@ class Buildozer:
                 # TODO more filters
                 basename, ext = splitext(fn)
                 if ext:
-                    ext = ext[1:]
+                    ext = ext[1:].lower()
                     if include_exts and ext not in include_exts:
                         continue
                     if exclude_exts and ext in exclude_exts:
@@ -816,10 +798,6 @@ class Buildozer:
         # copy also the libs
         copytree(self.applibs_dir, join(self.app_dir, '_applibs'))
 
-    def _copy_garden_libs(self):
-        if exists(self.gardenlibs_dir):
-            copytree(self.gardenlibs_dir, join(self.app_dir, 'libs'))
-
     def _add_sitecustomize(self):
         copyfile(join(dirname(__file__), 'sitecustomize.py'),
                 join(self.app_dir, 'sitecustomize.py'))
@@ -846,7 +824,7 @@ class Buildozer:
 
     @property
     def root_dir(self):
-        return realpath(dirname(self.specfilename))
+        return realpath(expanduser(dirname(self.specfilename)))
 
     @property
     def user_build_dir(self):
@@ -860,7 +838,7 @@ class Buildozer:
         build_dir = self.config.getdefault('buildozer', 'build_dir', build_dir)
 
         if build_dir is not None:
-            build_dir = realpath(join(self.root_dir, build_dir))
+            build_dir = realpath(join(self.root_dir, expanduser(build_dir)))
 
         return build_dir
 
@@ -889,10 +867,6 @@ class Buildozer:
     def applibs_dir(self):
         return join(self.buildozer_dir, 'applibs')
 
-    @property
-    def gardenlibs_dir(self):
-        return join(self.buildozer_dir, 'libs')
-
     @property
     def global_buildozer_dir(self):
         return join(expanduser('~'), '.buildozer')
@@ -961,8 +935,7 @@ class Buildozer:
 
             if not meth.__doc__:
                 continue
-            doc = [x for x in
-                    meth.__doc__.strip().splitlines()][0].strip()
+            doc = list(meth.__doc__.strip().splitlines())[0].strip()
             print('  {0:<18} {1}'.format(name, doc))
 
         print('')
