@@ -12,6 +12,7 @@ class Target:
     def __init__(self, buildozer):
         self.buildozer = buildozer
         self.build_mode = 'debug'
+        self.artifact_format = 'apk'
         self.platform_update = False
 
     def check_requirements(self):
@@ -101,6 +102,7 @@ class Target:
     def cmd_debug(self, *args):
         self.buildozer.prepare_for_build()
         self.build_mode = 'debug'
+        self.artifact_format = 'apk'
         self.buildozer.build()
 
     def cmd_release(self, *args):
@@ -137,6 +139,7 @@ class Target:
                 exit(1)
 
         self.build_mode = 'release'
+        self.artifact_format = self.buildozer.config.getdefault('app', 'android.release_artifact', 'aab')
         self.buildozer.build()
 
     def cmd_deploy(self, *args):
@@ -234,7 +237,7 @@ class Target:
 
         This will clone the contents of a git repository to
         `buildozer.platform_dir`. The location of this repo can be
-        speficied via URL and branch name, or via a custom (local)
+        specified via URL and branch name, or via a custom (local)
         directory name.
 
         :Parameters:
