@@ -7,7 +7,7 @@ if sys.platform == 'win32':
     raise NotImplementedError('Windows platform not yet working for Android')
 
 from platform import uname
-WSL = 'Microsoft' in uname()[2]
+WSL = 'microsoft' in uname()[2].lower()
 
 ANDROID_API = '27'
 ANDROID_MINAPI = '21'
@@ -33,6 +33,7 @@ from os.path import exists, join, realpath, expanduser, basename, relpath
 from platform import architecture
 from shutil import copyfile, rmtree
 from glob import glob
+from time import sleep
 
 from buildozer.libs.version import parse
 from distutils.version import LooseVersion
@@ -46,7 +47,7 @@ DEPRECATED_TOKENS = (('app', 'android.sdk'), )
 # does.
 DEFAULT_SDK_TAG = '6514223'
 
-DEFAULT_ARCH = 'armeabi-v7a'
+DEFAULT_ARCHS = ['arm64-v8a', 'armeabi-v7a']
 
 MSG_P4A_RECOMMENDED_NDK_ERROR = (
     "WARNING: Unable to find recommended Android NDK for current "
@@ -60,21 +61,29 @@ class TargetAndroid(Target):
     p4a_directory_name = "python-for-android"
     p4a_fork = 'kivy'
     p4a_branch = 'master'
-    p4a_apk_cmd = "apk --debug --bootstrap="
+    p4a_commit = 'HEAD'
     p4a_recommended_ndk_version = None
     extra_p4a_args = ''
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._arch = self.buildozer.config.getdefault(
-            'app', 'android.arch', DEFAULT_ARCH)
+        if self.buildozer.config.has_option(
+            "app", "android.arch"
+        ) and not self.buildozer.config.has_option("app", "android.archs"):
+            self.buildozer.error("`android.archs` not detected, instead `android.arch` is present.")
+            self.buildozer.error("`android.arch` will be removed and ignored in future.")
+            self.buildozer.error("If you're seeing this error, please migrate to `android.archs`.")
+            self._archs = self.buildozer.config.getlist(
+                'app', 'android.arch', DEFAULT_ARCHS)
+        else:
+            self._archs = self.buildozer.config.getlist(
+                'app', 'android.archs', DEFAULT_ARCHS)
         self._build_dir = join(
-            self.buildozer.platform_dir, 'build-{}'.format(self._arch))
+            self.buildozer.platform_dir, 'build-{}'.format(self.archs_snake))
         executable = sys.executable or 'python'
         self._p4a_cmd = '{} -m pythonforandroid.toolchain '.format(executable)
         self._p4a_bootstrap = self.buildozer.config.getdefault(
             'app', 'p4a.bootstrap', 'sdl2')
-        self.p4a_apk_cmd += self._p4a_bootstrap
         color = 'always' if USE_COLOR else 'never'
         self.extra_p4a_args = ' --color={} --storage-dir="{}"'.format(
             color, self._build_dir)
@@ -87,11 +96,30 @@ class TargetAndroid(Target):
 
         hook = self.buildozer.config.getdefault("app", "p4a.hook", None)
         if hook is not None:
-            self.extra_p4a_args += ' --hook={}'.format(realpath(hook))
+            self.extra_p4a_args += ' --hook={}'.format(realpath(expanduser(hook)))
         port = self.buildozer.config.getdefault('app', 'p4a.port', None)
         if port is not None:
             self.extra_p4a_args += ' --port={}'.format(port)
 
+        setup_py = self.buildozer.config.getdefault('app', 'p4a.setup_py', False)
+        if setup_py:
+            self.extra_p4a_args += ' --use-setup-py'
+        else:
+            self.extra_p4a_args += ' --ignore-setup-py'
+
+        activity_class_name = self.buildozer.config.getdefault(
+            'app', 'android.activity_class_name', 'org.kivy.android.PythonActivity')
+        if activity_class_name != 'org.kivy.android.PythonActivity':
+            self.extra_p4a_args += ' --activity-class-name={}'.format(activity_class_name)
+
+        if self.buildozer.log_level >= 2:
+            self.extra_p4a_args += ' --debug'
+
+        user_extra_p4a_args = self.buildozer.config.getdefault('app', 'p4a.extra_args',
+                                                               None)
+        if user_extra_p4a_args:
+            self.extra_p4a_args += ' ' + user_extra_p4a_args
+
         self.warn_on_deprecated_tokens()
 
     def warn_on_deprecated_tokens(self):
@@ -113,7 +141,7 @@ class TargetAndroid(Target):
         # Default p4a dir
         p4a_dir = join(self.buildozer.platform_dir, self.p4a_directory_name)
 
-        # Possibly overriden by user setting
+        # Possibly overridden by user setting
         system_p4a_dir = self.buildozer.config.getdefault('app', 'p4a.source_dir')
         if system_p4a_dir:
             p4a_dir = expanduser(system_p4a_dir)
@@ -227,6 +255,10 @@ class TargetAndroid(Target):
                  'installed'.format(sdkmanager_path)))
         return sdkmanager_path
 
+    @property
+    def archs_snake(self):
+        return "_".join(self._archs)
+
     def check_requirements(self):
         if platform in ('win32', 'cygwin'):
             try:
@@ -237,10 +269,7 @@ class TargetAndroid(Target):
                                 'adb.exe')
             self.javac_cmd = self._locate_java('javac.exe')
             self.keytool_cmd = self._locate_java('keytool.exe')
-        elif platform in ('darwin', ):
-            self.adb_cmd = join(self.android_sdk_dir, 'platform-tools', 'adb')
-            self.javac_cmd = self._locate_java('javac')
-            self.keytool_cmd = self._locate_java('keytool')
+        # darwin, linux
         else:
             self.adb_cmd = join(self.android_sdk_dir, 'platform-tools', 'adb')
             self.javac_cmd = self._locate_java('javac')
@@ -256,6 +285,12 @@ class TargetAndroid(Target):
                     'zlib headers must be installed, '
                     'run: sudo apt-get install zlib1g-dev')
 
+        # Adb arguments:
+        adb_args = self.buildozer.config.getdefault(
+            "app", "android.adb_args", None)
+        if adb_args is not None:
+            self.adb_cmd += ' ' + adb_args
+
         # Need to add internally installed ant to path for external tools
         # like adb to use
         path = [join(self.apache_ant_dir, 'bin')]
@@ -291,6 +326,13 @@ class TargetAndroid(Target):
 
         super().check_configuration_tokens(errors)
 
+    def _p4a_have_aab_support(self):
+        returncode = self._p4a("aab -h", break_on_error=False, show_output=False)[2]
+        if returncode == 0:
+            return True
+        else:
+            return False
+
     def _get_available_permissions(self):
         key = 'android:available_permissions'
         key_sdk = 'android:available_permissions_sdk'
@@ -358,7 +400,7 @@ class TargetAndroid(Target):
 
         self.buildozer.info('Android ANT is missing, downloading')
         archive = 'apache-ant-{0}-bin.tar.gz'.format(APACHE_ANT_VERSION)
-        url = 'http://archive.apache.org/dist/ant/binaries/'
+        url = 'https://archive.apache.org/dist/ant/binaries/'
         self.buildozer.download(url,
                                 archive,
                                 cwd=ant_dir)
@@ -406,7 +448,7 @@ class TargetAndroid(Target):
             return ndk_dir
 
         import re
-        _version = re.search('(.+?)[a-z]', self.android_ndk_version).group(1)
+        _version = int(re.search(r'(\d+)', self.android_ndk_version).group(1))
 
         self.buildozer.info('Android NDK is missing, downloading')
         # Welcome to the NDK URL hell!
@@ -416,28 +458,23 @@ class TargetAndroid(Target):
         # from 10e on the URLs can be looked up at
         # https://developer.android.com/ndk/downloads/older_releases
 
+        is_darwin = platform == 'darwin'
+        is_linux = platform.startswith('linux')
+
         if platform in ('win32', 'cygwin'):
-            # Checking of 32/64 bits at Windows from: http://stackoverflow.com/a/1405971/798575
+            # Checking of 32/64 bits at Windows from: https://stackoverflow.com/a/1405971/798575
             import struct
             archive = 'android-ndk-r{0}-windows-{1}.zip'
             is_64 = (8 * struct.calcsize("P") == 64)
-
-        elif platform in ('darwin', ):
-            if _version >= '10e':
-                archive = 'android-ndk-r{0}-darwin-{1}.zip'
-            elif _version >= '10c':
-                archive = 'android-ndk-r{0}-darwin-{1}.bin'
+        elif is_darwin or is_linux:
+            _platform = 'linux' if is_linux else 'darwin'
+            if self.android_ndk_version in ['10c', '10d', '10e']:
+                ext = 'bin'
+            elif _version <= 10:
+                ext = 'tar.bz2'
             else:
-                archive = 'android-ndk-r{0}-darwin-{1}.tar.bz2'
-            is_64 = (os.uname()[4] == 'x86_64')
-
-        elif platform.startswith('linux'):
-            if _version >= '10e':
-                archive = 'android-ndk-r{0}-linux-{1}.zip'
-            elif _version >= '10c':
-                archive = 'android-ndk-r{0}-linux-{1}.bin'
-            else:
-                archive = 'android-ndk-r{0}-linux-{1}.tar.bz2'
+                ext = 'zip'
+            archive = 'android-ndk-r{0}-' + _platform + '-{1}.' + ext
             is_64 = (os.uname()[4] == 'x86_64')
         else:
             raise SystemError('Unsupported platform: {}'.format(platform))
@@ -447,10 +484,10 @@ class TargetAndroid(Target):
         archive = archive.format(self.android_ndk_version, architecture)
         unpacked = unpacked.format(self.android_ndk_version)
 
-        if _version >= '10e':
+        if _version >= 11:
             url = 'https://dl.google.com/android/repository/'
         else:
-            url = 'http://dl.google.com/android/ndk/'
+            url = 'https://dl.google.com/android/ndk/'
 
         self.buildozer.download(url,
                                 archive,
@@ -649,7 +686,7 @@ class TargetAndroid(Target):
                 self.buildozer.error(
                     'You might have missed to install 32bits libs')
                 self.buildozer.error(
-                    'Check http://buildozer.readthedocs.org/en/latest/installation.html')
+                    'Check https://buildozer.readthedocs.org/en/latest/installation.html')
                 self.buildozer.error('')
             else:
                 self.buildozer.error('')
@@ -668,6 +705,12 @@ class TargetAndroid(Target):
         # ultimate configuration check.
         # some of our configuration cannot be check without platform.
         self.check_configuration_tokens()
+        if not self._p4a_have_aab_support():
+            self.buildozer.error(
+                "This buildozer version requires a python-for-android version with AAB (Android App Bundle) support. "
+                "Please update your pinned version accordingly."
+            )
+            raise BuildozerException()
 
         self.buildozer.environ.update({
             'PACKAGES_PATH': self.buildozer.global_packages_dir,
@@ -682,9 +725,15 @@ class TargetAndroid(Target):
         p4a_fork = self.buildozer.config.getdefault(
             'app', 'p4a.fork', self.p4a_fork
         )
+        p4a_url = self.buildozer.config.getdefault(
+            'app', 'p4a.url', f'https://github.com/{p4a_fork}/python-for-android.git'
+        )
         p4a_branch = self.buildozer.config.getdefault(
             'app', 'p4a.branch', self.p4a_branch
         )
+        p4a_commit = self.buildozer.config.getdefault(
+            'app', 'p4a.commit', self.p4a_commit
+        )
 
         p4a_dir = self.p4a_dir
         system_p4a_dir = self.buildozer.config.getdefault('app',
@@ -697,21 +746,19 @@ class TargetAndroid(Target):
                 self.buildozer.error('')
                 raise BuildozerException()
         else:
-            # check that fork/branch has not been changed
+            # check that url/branch has not been changed
             if self.buildozer.file_exists(p4a_dir):
-                cur_fork = cmd(
+                cur_url = cmd(
                     'git config --get remote.origin.url',
                     get_stdout=True,
                     cwd=p4a_dir,
-                )[0].split('/')[3]
+                )[0].strip()
                 cur_branch = cmd(
                     'git branch -vv', get_stdout=True, cwd=p4a_dir
                 )[0].split()[1]
-                if any([cur_fork != p4a_fork, cur_branch != p4a_branch]):
+                if any([cur_url != p4a_url, cur_branch != p4a_branch]):
                     self.buildozer.info(
-                        "Detected old fork/branch ({}/{}), deleting...".format(
-                            cur_fork, cur_branch
-                        )
+                        f"Detected old url/branch ({cur_url}/{cur_branch}), deleting..."
                     )
                     rmtree(p4a_dir)
 
@@ -719,11 +766,10 @@ class TargetAndroid(Target):
                 cmd(
                     (
                         'git clone -b {p4a_branch} --single-branch '
-                        'https://github.com/{p4a_fork}/python-for-android.git '
-                        '{p4a_dir}'
+                        '{p4a_url} {p4a_dir}'
                     ).format(
                         p4a_branch=p4a_branch,
-                        p4a_fork=p4a_fork,
+                        p4a_url=p4a_url,
                         p4a_dir=self.p4a_directory_name,
                     ),
                     cwd=self.buildozer.platform_dir,
@@ -738,6 +784,8 @@ class TargetAndroid(Target):
                     cmd('git fetch --tags origin {0}:{0}'.format(p4a_branch),
                         cwd=p4a_dir)
                     cmd('git checkout {}'.format(p4a_branch), cwd=p4a_dir)
+            if p4a_commit != 'HEAD':
+                cmd('git reset --hard {}'.format(p4a_commit), cwd=p4a_dir)
 
         # also install dependencies (currently, only setup.py knows about it)
         # let's extract them.
@@ -749,7 +797,7 @@ class TargetAndroid(Target):
         except IOError:
             self.buildozer.error('Failed to read python-for-android setup.py at {}'.format(
                 join(self.p4a_dir, 'setup.py')))
-            exit(1)
+            sys.exit(1)
         pip_deps = []
         for dep in deps:
             pip_deps.append("'{}'".format(dep))
@@ -785,33 +833,29 @@ class TargetAndroid(Target):
         if local_recipes:
             options.append('--local-recipes')
             options.append(local_recipes)
-        self._p4a(
-            ("create --dist_name={} --bootstrap={} --requirements={} "
-             "--arch {} {}").format(
-                 dist_name, self._p4a_bootstrap, requirements,
-                 self._arch, " ".join(options)),
-            get_stdout=True)[0]
+
+        p4a_create = "create --dist_name={} --bootstrap={} --requirements={} ".format(dist_name, self._p4a_bootstrap, requirements)
+
+        for arch in self._archs:
+            p4a_create += "--arch {} ".format(arch)
+
+        p4a_create += " ".join(options)
+
+        self._p4a(p4a_create, get_stdout=True)[0]
 
     def get_available_packages(self):
         return True
 
-    def get_dist_dir(self, dist_name, arch):
-        """Find the dist dir with the given name and target arch, if one
+    def get_dist_dir(self, dist_name):
+        """Find the dist dir with the given name if one
         already exists, otherwise return a new dist_dir name.
         """
-        expected_dist_name = generate_dist_folder_name(dist_name, arch_names=[arch])
 
         # If the expected dist name does exist, simply use that
-        expected_dist_dir = join(self._build_dir, 'dists', expected_dist_name)
+        expected_dist_dir = join(self._build_dir, 'dists', dist_name)
         if exists(expected_dist_dir):
             return expected_dist_dir
 
-        # For backwards compatibility, check if a directory without
-        # the arch exists. If so, this is probably the target dist.
-        old_dist_dir = join(self._build_dir, 'dists', dist_name)
-        if exists(old_dist_dir):
-            return old_dist_dir
-
         # If no directory has been found yet, our dist probably
         # doesn't exist yet, so use the expected name
         return expected_dist_dir
@@ -824,7 +868,7 @@ class TargetAndroid(Target):
         # wrapper from previous old_toolchain to new toolchain
         dist_name = self.buildozer.config.get('app', 'package.name')
         local_recipes = self.get_local_recipes_dir()
-        cmd = [self.p4a_apk_cmd, "--dist_name", dist_name]
+        cmd = [self.artifact_format, "--bootstrap", self._p4a_bootstrap, "--dist_name", dist_name]
         for args in build_cmd:
             option, values = args[0], args[1:]
             if option == "debug":
@@ -868,16 +912,27 @@ class TargetAndroid(Target):
         blacklist_src = self.buildozer.config.getdefault('app', 'android.blacklist_src', None)
         if whitelist_src:
             cmd.append('--whitelist')
-            cmd.append(realpath(whitelist_src))
+            cmd.append(realpath(expanduser(whitelist_src)))
         if blacklist_src:
             cmd.append('--blacklist')
-            cmd.append(realpath(blacklist_src))
+            cmd.append(realpath(expanduser(blacklist_src)))
 
         # support for aars
         aars = self.buildozer.config.getlist('app', 'android.add_aars', [])
         for aar in aars:
             cmd.append('--add-aar')
-            cmd.append(realpath(aar))
+            cmd.append(realpath(expanduser(aar)))
+
+        # support for assets folder
+        assets = self.buildozer.config.getlist('app', 'android.add_assets', [])
+        for asset in assets:
+            cmd.append('--add-asset')
+            if ':' in asset:
+                asset_src, asset_dest = asset.split(":")
+            else:
+                asset_src = asset
+                asset_dest = asset
+            cmd.append(realpath(expanduser(asset_src)) + ':' + asset_dest)
 
         # support for uses-lib
         uses_library = self.buildozer.config.getlist(
@@ -885,20 +940,58 @@ class TargetAndroid(Target):
         for lib in uses_library:
             cmd.append('--uses-library={}'.format(lib))
 
+        # support for activity-class-name
+        activity_class_name = self.buildozer.config.getdefault(
+            'app', 'android.activity_class_name', 'org.kivy.android.PythonActivity')
+        if activity_class_name != 'org.kivy.android.PythonActivity':
+            cmd.append('--activity-class-name={}'.format(activity_class_name))
+
+        # support for service-class-name
+        service_class_name = self.buildozer.config.getdefault(
+            'app', 'android.service_class_name', 'org.kivy.android.PythonService')
+        if service_class_name != 'org.kivy.android.PythonService':
+            cmd.append('--service-class-name={}'.format(service_class_name))
+
+        # support for extra-manifest-xml
+        extra_manifest_xml = self.buildozer.config.getdefault(
+            'app', 'android.extra_manifest_xml', '')
+        if extra_manifest_xml:
+            cmd.append('--extra-manifest-xml="{}"'.format(open(extra_manifest_xml, 'rt').read()))
+
+        # support for extra-manifest-application-arguments
+        extra_manifest_application_arguments = self.buildozer.config.getdefault(
+            'app', 'android.extra_manifest_application_arguments', '')
+        if extra_manifest_application_arguments:
+            args_body = open(extra_manifest_application_arguments, 'rt').read().replace('"', '\\"').replace('\n', ' ').replace('\t', ' ')
+            cmd.append('--extra-manifest-application-arguments="{}"'.format(args_body))
+
         # support for gradle dependencies
         gradle_dependencies = self.buildozer.config.getlist('app', 'android.gradle_dependencies', [])
         for gradle_dependency in gradle_dependencies:
             cmd.append('--depend')
             cmd.append(gradle_dependency)
 
-        cmd.append('--arch')
-        cmd.append(self._arch)
+        # support for manifestPlaceholders
+        manifest_placeholders = self.buildozer.config.getdefault('app', 'android.manifest_placeholders', None)
+        if manifest_placeholders:
+            cmd.append('--manifest-placeholders')
+            cmd.append("{}".format(manifest_placeholders))
+
+        # support disabling of compilation
+        compile_py = self.buildozer.config.getdefault('app', 'android.no-compile-pyo', None)
+        if compile_py:
+            cmd.append('--no-compile-pyo')
+
+        for arch in self._archs:
+            cmd.append('--arch')
+            cmd.append(arch)
 
         cmd = " ".join(cmd)
         self._p4a(cmd)
 
     def get_release_mode(self):
-        if self.check_p4a_sign_env():
+        # aab, also if unsigned is named as *-release
+        if self.check_p4a_sign_env() or self.artifact_format == "aab":
             return "release"
         return "release-unsigned"
 
@@ -940,6 +1033,12 @@ class TargetAndroid(Target):
                 cwd=self.buildozer.global_platform_dir)
         self.buildozer.environ.pop('ANDROID_SERIAL', None)
 
+        while True:
+            if self._get_pid():
+                break
+            sleep(.1)
+            self.buildozer.info('Waiting for application to start.')
+
         self.buildozer.info('Application started.')
 
     def cmd_p4a(self, *args):
@@ -983,8 +1082,7 @@ class TargetAndroid(Target):
 
     def build_package(self):
         dist_name = self.buildozer.config.get('app', 'package.name')
-        arch = self.buildozer.config.getdefault('app', 'android.arch', DEFAULT_ARCH)
-        dist_dir = self.get_dist_dir(dist_name, arch)
+        dist_dir = self.get_dist_dir(dist_name)
         config = self.buildozer.config
         package = self._get_package()
         version = self.buildozer.get_version()
@@ -1001,7 +1099,7 @@ class TargetAndroid(Target):
             patterns = config.getlist('app', config_key, [])
             if not patterns:
                 continue
-            if self._arch != lib_dir:
+            if lib_dir not in self._archs:
                 continue
 
             self.buildozer.debug('Search and copy libs for {}'.format(lib_dir))
@@ -1045,6 +1143,11 @@ class TargetAndroid(Target):
             permission = '.'.join(permission)
             build_cmd += [("--permission", permission)]
 
+        # add features
+        features = config.getlist('app', 'android.features', [])
+        for feature in features:
+            build_cmd += [("--feature", feature)]
+
         # android.entrypoint
         entrypoint = config.getdefault('app', 'android.entrypoint', 'org.kivy.android.PythonActivity')
         build_cmd += [('--android-entrypoint', entrypoint)]
@@ -1092,16 +1195,26 @@ class TargetAndroid(Target):
         for activity in add_activities:
             build_cmd += [("--add-activity", activity)]
 
-        # add presplash
-        presplash = config.getdefault('app', 'presplash.filename', '')
+        # add presplash, lottie animation or static
+        presplash = config.getdefault('app', 'android.presplash_lottie', '')
         if presplash:
-            build_cmd += [("--presplash", join(self.buildozer.root_dir,
-                                               presplash))]
+            build_cmd += [("--presplash-lottie", join(self.buildozer.root_dir,
+                                                      presplash))]
+        else:
+            presplash = config.getdefault('app', 'presplash.filename', '')
+            if presplash:
+                build_cmd += [("--presplash", join(self.buildozer.root_dir,
+                                                   presplash))]
 
         # add icon
         icon = config.getdefault('app', 'icon.filename', '')
         if icon:
             build_cmd += [("--icon", join(self.buildozer.root_dir, icon))]
+        icon_fg = config.getdefault('app', 'icon.adaptive_foreground.filename', '')
+        icon_bg = config.getdefault('app', 'icon.adaptive_background.filename', '')
+        if icon_fg and icon_bg:
+            build_cmd += [("--icon-fg", join(self.buildozer.root_dir, icon_fg))]
+            build_cmd += [("--icon-bg", join(self.buildozer.root_dir, icon_bg))]
 
         # OUYA Console support
         ouya_category = config.getdefault('app', 'android.ouya.category',
@@ -1135,6 +1248,13 @@ class TargetAndroid(Target):
         if wakelock:
             build_cmd += [("--wakelock", )]
 
+        # AndroidX ?
+        enable_androidx = config.getbooldefault('app',
+                                                'android.enable_androidx',
+                                                False)
+        if enable_androidx:
+            build_cmd += [("--enable-androidx", )]
+
         # intent filters
         intent_filters = config.getdefault(
             'app', 'android.manifest.intent_filters', '')
@@ -1153,6 +1273,17 @@ class TargetAndroid(Target):
         if numeric_version:
             build_cmd += [("--numeric-version", numeric_version)]
 
+        # android.allow_backup
+        allow_backup = config.getbooldefault('app', 'android.allow_backup', True)
+        if not allow_backup:
+            build_cmd += [('--allow-backup', 'false')]
+
+        # android.backup_rules
+        backup_rules = config.getdefault('app', 'android.backup_rules', '')
+        if backup_rules:
+            build_cmd += [("--backup-rules", join(self.buildozer.root_dir,
+                                                  backup_rules))]
+
         # build only in debug right now.
         if self.build_mode == 'debug':
             build_cmd += [("debug", )]
@@ -1184,9 +1315,12 @@ class TargetAndroid(Target):
         if is_gradle_build:
             # on gradle build, the apk use the package name, and have no version
             packagename_src = basename(dist_dir)  # gradle specifically uses the folder name
-            apk = u'{packagename}-{mode}.apk'.format(
-                packagename=packagename_src, mode=mode)
-            apk_dir = join(dist_dir, "build", "outputs", "apk", mode_sign)
+            artifact = u'{packagename}-{mode}.{artifact_format}'.format(
+                packagename=packagename_src, mode=mode, artifact_format=self.artifact_format)
+            if self.artifact_format == "apk":
+                artifact_dir = join(dist_dir, "build", "outputs", "apk", mode_sign)
+            elif self.artifact_format == "aab":
+                artifact_dir = join(dist_dir, "build", "outputs", "bundle", mode_sign)
         else:
             # on ant, the apk use the title, and have version
             bl = u'\'" ,'
@@ -1194,23 +1328,23 @@ class TargetAndroid(Target):
             if hasattr(apptitle, 'decode'):
                 apptitle = apptitle.decode('utf-8')
             apktitle = ''.join([x for x in apptitle if x not in bl])
-            apk = u'{title}-{version}-{mode}.apk'.format(
+            artifact = u'{title}-{version}-{mode}.apk'.format(
                 title=apktitle,
                 version=version,
                 mode=mode)
-            apk_dir = join(dist_dir, "bin")
+            artifact_dir = join(dist_dir, "bin")
 
-        apk_dest = u'{packagename}-{version}-{arch}-{mode}.apk'.format(
+        artifact_dest = u'{packagename}-{version}-{arch}-{mode}.{artifact_format}'.format(
             packagename=packagename, mode=mode, version=version,
-            arch=self._arch)
+            arch=self.archs_snake, artifact_format=self.artifact_format)
 
         # copy to our place
-        copyfile(join(apk_dir, apk), join(self.buildozer.bin_dir, apk_dest))
+        copyfile(join(artifact_dir, artifact), join(self.buildozer.bin_dir, artifact_dest))
 
         self.buildozer.info('Android packaging done!')
         self.buildozer.info(
-            u'APK {0} available in the bin directory'.format(apk_dest))
-        self.buildozer.state['android:latestapk'] = apk_dest
+            u'APK {0} available in the bin directory'.format(artifact_dest))
+        self.buildozer.state['android:latestapk'] = artifact_dest
         self.buildozer.state['android:latestmode'] = self.build_mode
 
     def _update_libraries_references(self, dist_dir):
@@ -1235,8 +1369,8 @@ class TargetAndroid(Target):
         # convert our references to relative path
         app_references = self.buildozer.config.getlist(
             'app', 'android.library_references', [])
-        source_dir = realpath(self.buildozer.config.getdefault(
-            'app', 'source.dir', '.'))
+        source_dir = realpath(expanduser(self.buildozer.config.getdefault(
+            'app', 'source.dir', '.')))
         for cref in app_references:
             # get the full path of the current reference
             ref = realpath(join(source_dir, cref))
@@ -1244,9 +1378,9 @@ class TargetAndroid(Target):
                 self.buildozer.error(
                     'Invalid library reference (path not found): {}'.format(
                         cref))
-                exit(1)
+                sys.exit(1)
             # get a relative path from the project file
-            ref = relpath(ref, realpath(dist_dir))
+            ref = relpath(ref, realpath(expanduser(dist_dir)))
             # ensure the reference exists
             references.append(ref)
 
@@ -1328,6 +1462,7 @@ class TargetAndroid(Target):
 
         if state.get('android:latestmode', '') != 'debug':
             self.buildozer.error('Only debug APK are supported for deploy')
+            return
 
         # search the APK in the bin dir
         apk = state['android:latestapk']
@@ -1347,6 +1482,18 @@ class TargetAndroid(Target):
 
         self.buildozer.info('Application pushed.')
 
+    def _get_pid(self):
+        pid, *_ = self.buildozer.cmd(
+            f'{self.adb_cmd} shell pidof {self._get_package()}',
+            get_stdout=True,
+            show_output=False,
+            break_on_error=False,
+            quiet=True,
+        )
+        if pid:
+            return pid.strip()
+        return False
+
     def cmd_logcat(self, *args):
         '''Show the log from the device
         '''
@@ -1358,38 +1505,26 @@ class TargetAndroid(Target):
             "app", "android.logcat_filters", "", section_sep=":", split_char=" ")
         filters = " ".join(filters)
         self.buildozer.environ['ANDROID_SERIAL'] = serial[0]
-        self.buildozer.cmd('{adb} logcat {filters}'.format(adb=self.adb_cmd,
-                                                           filters=filters),
-                           cwd=self.buildozer.global_platform_dir,
-                           show_output=True)
+        extra_args = []
+        pid = None
+        if self.buildozer.config.getdefault('app', 'android.logcat_pid_only'):
+            pid = self._get_pid()
+            if pid:
+                extra_args.extend(('--pid', pid))
+
+        self.buildozer.cmd(
+            f"{self.adb_cmd} logcat {filters} {' '.join(extra_args)}",
+            cwd=self.buildozer.global_platform_dir,
+            show_output=True,
+            run_condition=self._get_pid if pid else None,
+            break_on_error=False,
+        )
+
+        self.buildozer.info(f"{self._get_package()} terminated")
+
         self.buildozer.environ.pop('ANDROID_SERIAL', None)
 
 
 def get_target(buildozer):
     buildozer.targetname = "android"
     return TargetAndroid(buildozer)
-
-
-def generate_dist_folder_name(base_dist_name, arch_names=None):
-    """Generate the distribution folder name to use, based on a
-    combination of the input arguments.
-
-    WARNING: This function is copied from python-for-android. It would
-    be preferable to have a proper interface, either importing the p4a
-    code or having a p4a dist dir query option.
-
-    Parameters
-    ----------
-    base_dist_name : str
-        The core distribution identifier string
-    arch_names : list of str
-        The architecture compile targets
-
-    """
-    if arch_names is None:
-        arch_names = ["no_arch_specified"]
-
-    return '{}__{}'.format(
-        base_dist_name,
-        '_'.join(arch_names)
-    )
